package ch.psi.daq.rest.queries;

import java.util.LinkedHashSet;
import java.util.List;

import org.apache.commons.lang3.builder.ToStringBuilder;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;

import ch.psi.daq.cassandra.reader.Ordering;
import ch.psi.daq.hazelcast.query.Aggregation;
import ch.psi.daq.hazelcast.query.AggregationType;
import ch.psi.daq.hazelcast.query.bin.BinningStrategy;
import ch.psi.daq.hazelcast.query.range.QueryRange;

/**
 * 
 */
public class PulseRangeQuery extends AbstractQuery {


   /**
    * Constructor.
    * 
    * @param ordering whether to add a 'orderBy' clause into the database query
    * @param channels all the channelIds (channel names) we want to query
    * @param fields the fields (who map to fields in the DB) we are interested in returning to the
    *        client, needs to be in insertion order (hence the {@link LinkedHashSet} type)
    * @param binningStrategyEnum enum that maps the user's String to a concrete
    *        {@link BinningStrategy} implementation
    * @param binLengthOrCount depending on the chosen binning strategy, this field defines either the
    *        count (how many pulse ids are to be put inside 1 bin) or the time frame for 1 bin
    * @param aggregateChannels whether aggregation will include all channels, default is on a
    *        per-channel basis
    * @param aggregationType defines whether aggregation takes place in an index- or value-based
    *        manner
    * @param aggregations list of aggregations / statistics to calculate, e.g. min, max and average
    * @param queryRange object containing the ranges for either pulse-based queries or time-based
    *        queries
    */
   @JsonCreator
   public PulseRangeQuery(
         // note that those annotations are needed for the polymorphic
         // mapping to work correctly
         @JsonProperty(value = "ordering") Ordering ordering,
         @JsonProperty(value = "channels") List<String> channels,
         @JsonProperty(value = "fields") LinkedHashSet<String> fields,
         @JsonProperty(value = "binningStrategy") BinningStrategyEnum binningStrategyEnum,
         @JsonProperty(value = "binDuration") long binLengthOrCount,
         @JsonProperty(value = "aggregateChannels") boolean aggregateChannels,
         @JsonProperty(value = "aggregationType") AggregationType aggregationType,
         @JsonProperty(value = "aggregations") List<Aggregation> aggregations,
         @JsonProperty(value = "queryRange") QueryRange queryRange) {

      super(ordering, channels, fields, binningStrategyEnum, binLengthOrCount, aggregateChannels, aggregationType,
            aggregations, queryRange);

   }

   /**
    * {@inheritDoc}
    */
   @Override
   public String toString() {
      return ToStringBuilder.reflectionToString(this);
   }

}
