package ch.psi.daq.test.rest.controller;

import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;

import org.junit.Test;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.test.web.servlet.result.MockMvcResultHandlers;
import org.springframework.test.web.servlet.result.MockMvcResultMatchers;

import ch.psi.daq.cassandra.reader.Ordering;
import ch.psi.daq.hazelcast.query.AggregationType;
import ch.psi.daq.hazelcast.query.range.QueryRange;
import ch.psi.daq.hazelcast.query.range.QueryRangeImpl;
import ch.psi.daq.rest.queries.BinningStrategyEnum;
import ch.psi.daq.rest.queries.PulseRangeQuery;
import ch.psi.daq.rest.queries.TimeRangeQuery;
import ch.psi.daq.test.rest.AbstractDaqRestTest;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;

/**
 * Tests the {@link DaqController} implementation. 
 */
public class DaqRestControllerTest extends AbstractDaqRestTest {

   private static final List<String> DEFAULT_PROPERTIES = Lists.newArrayList("channel", "pulseId");

   @Test
   public void testPulseRangeQuery() throws Exception {
      QueryRange range = new QueryRangeImpl(100l, 101l);
      PulseRangeQuery request = new PulseRangeQuery(
            Ordering.desc, //ordering
            Lists.newArrayList(), // channels, DummyQueryProcessor simply returns a fixed list
            Sets.newLinkedHashSet(DEFAULT_PROPERTIES), // fields
            BinningStrategyEnum.count,
            100,
            false,
            AggregationType.index,
            null,
            range);
      
      String content = mapper.writeValueAsString(request);

      this.mockMvc
         .perform(MockMvcRequestBuilders.post("/pulserange")
               .contentType(MediaType.APPLICATION_JSON)
               .content(content))
         
               .andDo(MockMvcResultHandlers.print())
               .andExpect(MockMvcResultMatchers.status().isOk())
               .andExpect(MockMvcResultMatchers.jsonPath("$").isArray())
               .andExpect(MockMvcResultMatchers.jsonPath("$[0]").exists())
               .andExpect(MockMvcResultMatchers.jsonPath("$[0].pulseId").value(100))
               .andExpect(MockMvcResultMatchers.jsonPath("$[0].channel").value("testChannel1"));
   }
   
   @Test
   public void testTimeRangeQuery() throws Exception {
      
      long startTime = new Date().getTime();
      long endTime = startTime + TimeUnit.SECONDS.toMillis(1);
      QueryRange range = new QueryRangeImpl(startTime, 0, endTime, 0);
      TimeRangeQuery request = new TimeRangeQuery(
            Ordering.asc,
            Lists.newArrayList("test"),
            Sets.newLinkedHashSet(DEFAULT_PROPERTIES),
            BinningStrategyEnum.count,
            100,
            false,
            AggregationType.index,
            null, // aggregations
            range,
            null, // startMillis
            null);
      
      String content = mapper.writeValueAsString(request);
      System.out.println(content);
      
      this.mockMvc
      .perform(MockMvcRequestBuilders.post("/timerange")
            .contentType(MediaType.APPLICATION_JSON)
            .content(content))
            
            .andDo(MockMvcResultHandlers.print())
            .andExpect(MockMvcResultMatchers.status().isOk())
            .andExpect(MockMvcResultMatchers.jsonPath("$").isArray())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0]").exists())
            .andExpect(MockMvcResultMatchers.jsonPath("$[0].channel").value("testChannel1"));
   }
   
   @Test
   public void testMapper() throws JsonProcessingException {
      long startTime = new Date().getTime();
      long endTime = startTime + TimeUnit.SECONDS.toMillis(1);
      QueryRange range = new QueryRangeImpl(startTime, 0, endTime, 0);
      TimeRangeQuery request = new TimeRangeQuery(
            Ordering.asc,
            Lists.newArrayList("test"),
            Sets.newLinkedHashSet(DEFAULT_PROPERTIES),
            BinningStrategyEnum.count,
            100,
            false,
            AggregationType.index,
            null, // aggregations
            range,
            null, // startMillis
            null);
      
      String content = mapper.writeValueAsString(request);
      System.out.println(content);
   }
}
