package ch.psi.daq.test.rest.queries;

import java.util.Date;
import java.util.concurrent.TimeUnit;

import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.BlockJUnit4ClassRunner;

import ch.psi.daq.cassandra.reader.Ordering;
import ch.psi.daq.hazelcast.query.AggregationType;
import ch.psi.daq.hazelcast.query.bin.BinningStrategyBinCount;
import ch.psi.daq.hazelcast.query.bin.BinningStrategyLengthPulse;
import ch.psi.daq.hazelcast.query.bin.BinningStrategyLengthTime;
import ch.psi.daq.hazelcast.query.range.QueryRange;
import ch.psi.daq.hazelcast.query.range.QueryRangeImpl;
import ch.psi.daq.rest.queries.BinningStrategyEnum;
import ch.psi.daq.rest.queries.TimeRangeQuery;

import com.google.common.collect.Lists;
import com.google.common.collect.Sets;

@RunWith(BlockJUnit4ClassRunner.class)
public class AbstractQueryTestTest extends AbstractQueryTest {
   
   @Test(expected=IllegalArgumentException.class)
   public void test_NoChannels_exception() {
      
      long startMillis = new Date().getTime();
      long endMillis = new Date().getTime() + TimeUnit.SECONDS.toMillis(10);
      
      QueryRange range = new QueryRangeImpl(startMillis, 0, endMillis, 0);
      
      new TimeRangeQuery(
            Ordering.asc,
            null, // should throw exception
            Sets.newLinkedHashSet(DEFAULT_PROPERTIES),
            BinningStrategyEnum.count,
            100,             // binDurationOrBincount : long
            false,           // isAggregateChannels
            AggregationType.index,
            null,            // list of aggregations
            range,
            null,            // startDateTime : String
            null);           // endDateTime : String
      
   }
   
   @Test(expected=IllegalArgumentException.class)
   public void test_NoFields_exception() {
      
      long startMillis = new Date().getTime();
      long endMillis = new Date().getTime() + TimeUnit.SECONDS.toMillis(10);
      QueryRange range = new QueryRangeImpl(startMillis, 0, endMillis, 0);
      
      new TimeRangeQuery(
            Ordering.asc,
            Lists.newArrayList(),
            null,
            BinningStrategyEnum.count,
            100,             // binDurationOrBincount : long
            false,           // isAggregateChannels
            AggregationType.index,
            null,            // list of aggregations
            range, 
            null,            // startDateTime : String
            null);           // endDateTime : String
      
   }
   
   @Test
   public void test_nullBinningStrategyEnum_exception() {
      
      long startMillis = new Date().getTime();
      long endMillis = new Date().getTime() + TimeUnit.SECONDS.toMillis(10);
      QueryRange range = new QueryRangeImpl(startMillis, 0, endMillis, 0);
      TimeRangeQuery query = new TimeRangeQuery(
            Ordering.asc,
            Lists.newArrayList(),
            Sets.newLinkedHashSet(DEFAULT_PROPERTIES),
            null,
            100,             // binDurationOrBincount : long
            false,           // isAggregateChannels
            AggregationType.index,
            null,            // list of aggregations
            range, 
            null,            // startDateTime : String
            null);           // endDateTime : String
      
      Assert.assertTrue(query.getBinningStrategy() != null);
      Assert.assertEquals(BinningStrategyBinCount.class, query.getBinningStrategy().getClass());
   }
   
   @Test
   public void test_LengthBinningTimeStrategy() {
      
      long startMillis = new Date().getTime();
      long endMillis = new Date().getTime() + TimeUnit.SECONDS.toMillis(10);
      QueryRange range = new QueryRangeImpl(startMillis, 0, endMillis, 0);
      TimeRangeQuery query = new TimeRangeQuery(
            Ordering.asc,
            Lists.newArrayList(),
            Sets.newLinkedHashSet(DEFAULT_PROPERTIES),
            BinningStrategyEnum.length,
            100,             // binDurationOrBincount : long
            false,           // isAggregateChannels
            AggregationType.index,
            null,            // list of aggregations
            range, 
            null,            // startDateTime : String
            null);           // endDateTime : String
      
      Assert.assertTrue(query.getBinningStrategy() != null);
      Assert.assertEquals(BinningStrategyLengthTime.class, query.getBinningStrategy().getClass());
   }
   
   @Test
   public void test_LengthBinningPulseStrategy() {
      
      long startPulse = 100l;
      long endPulse = 105l;
      QueryRange range = new QueryRangeImpl(startPulse, endPulse);
      TimeRangeQuery query = new TimeRangeQuery(
            Ordering.asc,
            Lists.newArrayList(),
            Sets.newLinkedHashSet(DEFAULT_PROPERTIES),
            BinningStrategyEnum.length,
            100,             // binDurationOrBincount : long
            false,           // isAggregateChannels
            AggregationType.index,
            null,            // list of aggregations
            range, 
            null,            // startDateTime : String
            null);           // endDateTime : String
      
      Assert.assertTrue(query.getBinningStrategy() != null);
      Assert.assertEquals(BinningStrategyLengthPulse.class, query.getBinningStrategy().getClass());
   }
   
   @Test
   public void test_BinCountBinningStrategy() {
      
      long startMillis = new Date().getTime();
      long endMillis = new Date().getTime() + TimeUnit.SECONDS.toMillis(10);
      QueryRange range = new QueryRangeImpl(startMillis, 0, endMillis, 0);
      TimeRangeQuery query = new TimeRangeQuery(
            Ordering.asc,
            Lists.newArrayList(),
            Sets.newLinkedHashSet(DEFAULT_PROPERTIES),
            BinningStrategyEnum.count,
            100,             // binDurationOrBincount : long
            false,           // isAggregateChannels
            AggregationType.index,
            null,            // list of aggregations
            range, 
            null,            // startDateTime : String
            null);           // endDateTime : String
      
      Assert.assertTrue(query.getBinningStrategy() != null);
      Assert.assertEquals(BinningStrategyBinCount.class, query.getBinningStrategy().getClass());
   }
   
}
