package ch.psi.daq.test.rest.queries;

import java.text.ParseException;
import java.text.SimpleDateFormat;

import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.BlockJUnit4ClassRunner;

import ch.psi.daq.cassandra.reader.Ordering;
import ch.psi.daq.hazelcast.query.AggregationType;
import ch.psi.daq.hazelcast.query.range.QueryRange;
import ch.psi.daq.hazelcast.query.range.QueryRangeImpl;
import ch.psi.daq.rest.queries.BinningStrategyEnum;
import ch.psi.daq.rest.queries.TimeRangeQuery;

import com.google.common.collect.Lists;
import com.google.common.collect.Sets;

@RunWith(BlockJUnit4ClassRunner.class)
public class TimeRangeQueryTest extends AbstractQueryTest {
   
   @Test
   public void test_startAndEndDateTime_asString() throws ParseException {
      
      // allowed format: yyyy/MM/dd hh:mm:ss.SSS
      String startDateTime = "2014/01/01 12:00:00.000";
      String endDateTime = "2014/01/01 18:00:00.000";
      
      QueryRange range = new QueryRangeImpl(-1, 0, -1, 0);
      
      TimeRangeQuery query = new TimeRangeQuery(
            Ordering.asc,
            Lists.newArrayList(), // DummyQueryProcessor simply returns a fixed list
            Sets.newLinkedHashSet(DEFAULT_PROPERTIES),
            BinningStrategyEnum.count,
            100,             // binDurationOrBincount : long
            false,           // isAggregateChannels
            AggregationType.index,
            null,            // list of aggregations
            range,
            startDateTime,            // startDateTime : String
            endDateTime);           // endDateTime : String
      
      SimpleDateFormat format = new SimpleDateFormat(TimeRangeQuery.DATE_FORMAT_STRING);
      
      Assert.assertEquals(format.parse(startDateTime).getTime(), query.getQueryRange().getStartMillis());
      Assert.assertEquals(format.parse(endDateTime).getTime(), query.getQueryRange().getEndMillis());
   }
   
   @Test
   public void test_wrongFormat_exception() {
      String startDateTime = "2014-01-01 12:00:00.000";
      String endDateTime = "2014/01/01 18:00:00.000";
      QueryRange range = new QueryRangeImpl(-1, 0, -1, 0);
      TimeRangeQuery query = new TimeRangeQuery(
            Ordering.asc,
            Lists.newArrayList(), // DummyQueryProcessor simply returns a fixed list
            Sets.newLinkedHashSet(DEFAULT_PROPERTIES),
            BinningStrategyEnum.count,
            100,             // binDurationOrBincount : long
            false,           // isAggregateChannels
            AggregationType.index,
            null,
            range,
            startDateTime,            // startDateTime : String
            endDateTime);           // endDateTime : String
      
      Assert.assertEquals(-1, query.getQueryRange().getStartMillis());
      Assert.assertEquals(-1, query.getQueryRange().getStartMillis());
   }
}
